'use strict';

var path = require('path');
var schemaUtils = require('schema-utils');
var fs = require('fs');
var fsExtra = require('fs-extra');
var cpy = require('cpy');
var del = require('del');
var archiver = require('archiver');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
var fsExtra__default = /*#__PURE__*/_interopDefaultLegacy(fsExtra);
var cpy__default = /*#__PURE__*/_interopDefaultLegacy(cpy);
var del__default = /*#__PURE__*/_interopDefaultLegacy(del);
var archiver__default = /*#__PURE__*/_interopDefaultLegacy(archiver);

/*!
 * is-extglob <https://github.com/jonschlinkert/is-extglob>
 *
 * Copyright (c) 2014-2016, Jon Schlinkert.
 * Licensed under the MIT License.
 */
var isExtglob = function isExtglob(str) {
  if (typeof str !== 'string' || str === '') {
    return false;
  }

  var match;

  while (match = /(\\).|([@?!+*]\(.*\))/g.exec(str)) {
    if (match[2]) return true;
    str = str.slice(match.index + match[0].length);
  }

  return false;
};

/*!
 * is-glob <https://github.com/jonschlinkert/is-glob>
 *
 * Copyright (c) 2014-2017, Jon Schlinkert.
 * Released under the MIT License.
 */

var chars = {
  '{': '}',
  '(': ')',
  '[': ']'
};
var strictRegex = /\\(.)|(^!|\*|[\].+)]\?|\[[^\\\]]+\]|\{[^\\}]+\}|\(\?[:!=][^\\)]+\)|\([^|]+\|[^\\)]+\))/;
var relaxedRegex = /\\(.)|(^!|[*?{}()[\]]|\(\?)/;

var isGlob = function isGlob(str, options) {
  if (typeof str !== 'string' || str === '') {
    return false;
  }

  if (isExtglob(str)) {
    return true;
  }

  var regex = strictRegex;
  var match; // optionally relax regex

  if (options && options.strict === false) {
    regex = relaxedRegex;
  }

  while (match = regex.exec(str)) {
    if (match[2]) return true;
    var idx = match.index + match[0].length; // if an open bracket/brace/paren is escaped,
    // set the index to the next closing character

    var open = match[1];
    var close = open ? chars[open] : null;

    if (open && close) {
      var n = str.indexOf(close, idx);

      if (n !== -1) {
        idx = n + 1;
      }
    }

    str = str.slice(idx);
  }

  return false;
};

const copyAction = async tasks => {
  const taskMap = tasks.map(copy);
  await Promise.all(taskMap);
};

const copy = async task => {
  const {
    source,
    absSource,
    absDestination,
    context
  } = task;

  try {
    if (isGlob(source)) {
      const src = path__default['default'].posix.join(context, source);
      await cpy__default['default'](src, absDestination);
    } else {
      const isFolderTarget = /(?:\\|\/)$/.test(absDestination);
      const isSourceFile = fs__default['default'].lstatSync(absSource).isFile(); // if source is a file and target is a directory
      // create the directory and copy the file into that directory

      if (isSourceFile && isFolderTarget) {
        await fsExtra__default['default'].ensureDir(absDestination);
        const sourceFileName = path__default['default'].basename(absSource);
        const filePath = path__default['default'].resolve(absDestination, sourceFileName);
        await fsExtra__default['default'].copy(absSource, filePath);
        return;
      }

      await fsExtra__default['default'].copy(absSource, absDestination);
    }
  } catch (err) {}
};

const moveAction = async input => {
  for (const task of input) {
    await fsExtra__default['default'].move(task.absSource, task.absDestination);
  }
};

const deleteAction = async tasks => {
  for (const task of tasks) {
    await del__default['default'](task.absSource);
  }
};

const mkdirAction = async tasks => {
  for (const task of tasks) {
    await fsExtra__default['default'].ensureDir(task.absSource);
  }
};

const archiveAction = async tasks => {
  const actionMap = tasks.map(archive);
  await Promise.all(actionMap);
};

const archive = async task => {
  const {
    source,
    absSource,
    absDestination,
    options = {},
    context
  } = task;
  const format = task.format || path__default['default'].extname(absDestination).replace('.', ''); // Exclude destination file from archive

  const destFile = path__default['default'].basename(absDestination);
  const destDir = path__default['default'].dirname(absDestination);
  const globOptions = Object.assign({
    ignore: destFile
  }, options.globOptions || {});
  await fsExtra__default['default'].ensureDir(destDir);
  const output = fs__default['default'].createWriteStream(absDestination);
  const archive = archiver__default['default'](format, options);
  archive.pipe(output);

  if (isGlob(source)) {
    const gOptions = { ...globOptions,
      cwd: context
    };
    await archive.glob(source, gOptions).finalize();
  } else {
    const sStat = fs__default['default'].lstatSync(absSource);

    if (sStat.isDirectory()) {
      const gOptions = { ...globOptions,
        cwd: absSource
      };
      await archive.glob('**/*', gOptions).finalize();
    }

    if (sStat.isFile()) {
      const options = {
        name: path__default['default'].basename(source)
      };
      await archive.file(absSource, options).finalize();
    }
  } // output.close()

};

var schema = {
  title: 'FileManagerPluginOptions',
  type: 'object',
  additionalProperties: false,
  definitions: {
    Copy: {
      description: 'Copy individual files or entire directories from a source folder to a destination folder',
      type: 'array',
      minItems: 1,
      additionalItems: true,
      itmes: [{
        type: 'object',
        additionalProperties: false,
        properties: {
          source: {
            description: 'Copy source. A file or directory or a glob',
            type: 'string',
            minLength: 1
          },
          destination: {
            description: 'Copy destination',
            type: 'string',
            minLength: 1
          }
        }
      }]
    },
    Delete: {
      description: 'Delete individual files or entire directories',
      type: 'array',
      minItems: 1,
      additionalItems: true,
      items: {
        type: 'string'
      }
    },
    Move: {
      description: 'Move individual files or entire directories from a source folder to a destination folder',
      type: 'array',
      additionalItems: true,
      items: [{
        type: 'object',
        additionalProperties: false,
        properties: {
          source: {
            description: 'Move source. A file or directory or a glob',
            type: 'string',
            minLength: 1
          },
          destination: {
            description: 'Move destination',
            type: 'string',
            minLength: 1
          }
        }
      }]
    },
    Mkdir: {
      description: 'Create Directories',
      type: 'array',
      minItems: 1,
      additionalItems: true,
      items: {
        type: 'string'
      }
    },
    Archive: {
      description: 'Archive individual files or entire directories.',
      type: 'array',
      additionalItems: true,
      items: [{
        type: 'object',
        additionalProperties: false,
        properties: {
          source: {
            description: 'Source. A file or directory or a glob',
            type: 'string',
            minLength: 1
          },
          destination: {
            description: 'Archive destination',
            type: 'string',
            minLength: 1
          },
          format: {
            type: 'string',
            enum: ['zip', 'tar']
          },
          options: {
            additionalProperties: true,
            type: 'object',
            description: 'Options to forward to archiver'
          }
        }
      }]
    },
    Events: {
      type: 'object',
      additionalProperties: true,
      properties: {
        copy: {
          $ref: '#/definitions/Copy'
        },
        delete: {
          $ref: '#/definitions/Delete'
        },
        move: {
          $ref: '#/definitions/Move'
        },
        mkdir: {
          $ref: '#/definitions/Mkdir'
        },
        archive: {
          $ref: '#/definitions/Archive'
        }
      }
    }
  },
  properties: {
    onStart: {
      $ref: '#/definitions/Events'
    },
    onEnd: {
      $ref: '#/definitions/Events'
    }
  }
};

const PLUGIN_NAME = 'FileManagerPlugin';

const resolvePaths = (stages, context) => {
  Object.keys(stages).forEach(stage => {
    Object.keys(stages[stage]).forEach(action => {
      stages[stage][action] = stages[stage][action].map(task => {
        const source = typeof task === 'string' ? task : task.source;

        if (typeof task === 'string') {
          return {
            source,
            absSource: path__default['default'].isAbsolute(source) ? source : path__default['default'].join(context, source)
          };
        }

        const {
          destination
        } = task;
        return { ...task,
          source: source,
          absSource: path__default['default'].isAbsolute(source) ? source : path__default['default'].join(context, source),
          destination: destination,
          absDestination: path__default['default'].isAbsolute(destination) ? destination : path__default['default'].join(context, destination),
          context
        };
      });
    });
  });
  return stages;
};

class FileManagerPlugin {
  constructor(actions) {
    schemaUtils.validate(schema, actions, {
      name: PLUGIN_NAME,
      baseDataPath: 'actions'
    });
    this.actions = actions;
  }

  async processAction(action, actionParams) {
    const options = {
      context: this.context
    };
    await action(actionParams, options);
  }

  async execute(stage) {
    if (!this.actions[stage]) {
      return;
    }

    const stages = Object.keys(this.actions[stage]);
    const executionPromises = stages.map(async actionType => {
      const actionParams = this.actions[stage][actionType];

      switch (actionType) {
        case 'delete':
          return this.processAction(deleteAction, actionParams);

        case 'mkdir':
          return this.processAction(mkdirAction, actionParams);

        case 'copy':
          return this.processAction(copyAction, actionParams);

        case 'move':
          return this.processAction(moveAction, actionParams);

        case 'archive':
          return this.processAction(archiveAction, actionParams);

        default:
          return;
      }
    });
  }

  apply(compiler) {
    this.context = compiler.options.context;
    this.actions = resolvePaths(this.actions, this.context);

    const onStart = async () => {
      await this.execute('onStart');
    };

    const onEnd = async () => {
      await this.execute('onEnd');
    };

    compiler.hooks.beforeRun.tapPromise(PLUGIN_NAME, onStart);
    compiler.hooks.afterEmit.tapPromise(PLUGIN_NAME, onEnd);
  }

}

module.exports = FileManagerPlugin;
//# sourceMappingURL=index.js.map
